'use strict';

const url = require('url');
const load = require('@proload/core');
const promises = require('fs/promises');
const fg = require('fast-glob');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e["default"] : e; }

const load__default = /*#__PURE__*/_interopDefaultLegacy(load);
const fg__default = /*#__PURE__*/_interopDefaultLegacy(fg);

async function getUserConfig(root, configPath) {
  const resolvedRoot = url.fileURLToPath(new URL(root.href).href);
  let userConfigPath;
  if (configPath) {
    const configPathWithLeadingSlash = /^\.*\//.test(configPath) ? configPath : `./${configPath}`;
    userConfigPath = url.fileURLToPath(new URL(configPathWithLeadingSlash, root));
  }
  const astro = "astro.i18n";
  return await load__default(astro, { mustExist: false, cwd: resolvedRoot, filePath: userConfigPath });
}
const deeplyStringifyObject = (obj) => {
  const isArray = Array.isArray(obj);
  let str = isArray ? "[" : "{";
  for (const key in obj) {
    if (obj[key] === null || obj[key] === void 0) {
      continue;
    }
    let value = null;
    switch (typeof obj[key]) {
      case "string": {
        value = `"${obj[key]}"`;
        break;
      }
      case "number":
      case "boolean": {
        value = obj[key];
        break;
      }
      case "object": {
        value = deeplyStringifyObject(obj[key]);
        break;
      }
      case "function": {
        value = obj[key].toString().replace(/\s+/g, " ");
        break;
      }
      case "symbol": {
        value = `Symbol("${obj[key].description}")`;
        break;
      }
    }
    str += isArray ? `${value},` : `"${key}": ${value},`;
  }
  return `${str}${isArray ? "]" : "}"}`;
};

async function I18nCore(config, customConfigPath) {
  const userConfig = await getUserConfig(config.root, customConfigPath);
  if (!userConfig?.value) {
    throw new Error(
      `[astrojs-i18n]: Could not read a config file at ${JSON.stringify(
        customConfigPath
      )}`
    );
  }
  const astrojsI18nconfig = userConfig?.value;
  if (astrojsI18nconfig.defaultLanguage === "" || !astrojsI18nconfig.defaultLanguage) {
    throw new Error(
      `[astrojs-i18n]: you must set a baseLanguage in your astrojsI18nconfig`
    );
  }
  if (astrojsI18nconfig.supportedLanguages.length === 0) {
    astrojsI18nconfig.supportedLanguages = [...astrojsI18nconfig.defaultLanguage];
  }
  if (!astrojsI18nconfig.supportedLanguages.includes(astrojsI18nconfig.defaultLanguage)) {
    astrojsI18nconfig.supportedLanguages.unshift(astrojsI18nconfig.defaultLanguage);
  }
  if (!astrojsI18nconfig.localesPath || !astrojsI18nconfig.localesPath.loadPath) {
    throw new Error(
      "[astrojs-i18n]: you must set a `localesPath.loadPath` in your astrojsI18nconfig"
    );
  }
  astrojsI18nconfig.i18n = {
    ...astrojsI18nconfig.i18n,
    "supportedLngs": [...astrojsI18nconfig.supportedLanguages],
    "fallbackLng": [...astrojsI18nconfig.supportedLanguages]
  };
  const cwd = url.fileURLToPath(new URL(config.root.href).href);
  const filesRoot = astrojsI18nconfig.localesPath.loadPath;
  const files = await fg__default(filesRoot, {
    cwd,
    absolute: true,
    dot: true
  });
  let imports = `import i18next from 'i18next'
`;
  let i18n = `i18next`;
  astrojsI18nconfig.i18n.resources = {};
  if (files.length > 0) {
    for (const file of files) {
      const arrFile = file.split("/");
      const data = await promises.readFile(file, { encoding: "utf-8" });
      const nameFile = arrFile[arrFile.length - 1].split(".")[0];
      console.log(nameFile, "-----");
      astrojsI18nconfig.i18n.resources[nameFile] = {
        translation: JSON.parse(data)
      };
    }
  }
  i18n += `.init(${deeplyStringifyObject(
    astrojsI18nconfig.i18n
  )})`;
  return Promise.resolve(imports + i18n);
}

function index(options) {
  const customConfigPath = options?.configPath;
  return {
    name: "astrojs-i18n",
    hooks: {
      "astro:config:setup": async ({ config, injectScript }) => {
        const importi18n = await I18nCore(config, customConfigPath);
        injectScript("page-ssr", importi18n);
      }
    }
  };
}

module.exports = index;
